// Details about connecting LEDs to the GPIOs on the project page:
// http://homofaciens.de/technics-physical-computing-switching-LEDs_en.htm
//
// Switching LEDs with 28 GPIOs of a Raspberry Pi (tested with model 3)
// compile with: gcc LED.c -o LED -I/usr/local/include -L/usr/local/lib -lwiringPi -lpthread -lm


#include <wiringPi.h>

#include <stdio.h>


//Pin numbers of the 28 GPIOs used by wiringPi are not in order !!!!
int pinNr[]={0,1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31};


//######################################################################
//######################################################################
//######################################################################
int main(int argc, char **argv){
  int i, j, k;
  long uPause = 100000;


  if (wiringPiSetup () == -1)
    return (1) ;


  printf ("\nInitializing GPIOs as output.\n");
  for(i=0; i < 28; i++){
    pinMode (pinNr[i], OUTPUT);
  }


  while(1){


/*
      digitalWrite(28, 1);
      sleep(10);
      digitalWrite(27, 1);
      sleep(10);
      digitalWrite(29, 1);
      sleep(10);
      digitalWrite(28, 0);
      digitalWrite(27, 0);
      digitalWrite(29, 0);
      sleep(10);
*/

   //Turn LEDs ON in sequence
   uPause = 28000;
    sleep(10);
    for(i=0; i<28; i++){
      digitalWrite(pinNr[i], 1);
      usleep(uPause);
    }
    usleep(3);

   //Turn a single LED OFF for uPause 1000000µs = 1s
    for(i=0; i<28; i++){
      digitalWrite(pinNr[i], 0);
      usleep(uPause);
      digitalWrite(pinNr[i], 1);
      //usleep(uPause);
    }

    for(i=27; i>-1; i--){
      digitalWrite(pinNr[i], 0);
      usleep(uPause);
      digitalWrite(pinNr[i], 1);
      //usleep(uPause);
    }

   //Turn LEDs OFF in sequence
    for(i=0; i<28; i++){
      digitalWrite(pinNr[i], 0);
      usleep(uPause);
    }
    usleep(3);

   //Turn LEDs ON in sequence
    for(i=27; i>-1; i--){
      digitalWrite(pinNr[i], 1);
      usleep(uPause);
    }
    usleep(3);

   //Turn LEDs OFF in sequence
    for(i=27; i>-1; i--){
      digitalWrite(pinNr[i], 0);
      usleep(uPause);
    }
    usleep(3);

   //Turn a single LED ON for uPause 1000000µs = 1s
    for(i=0; i<28; i++){
      digitalWrite(pinNr[i], 1);
      usleep(uPause);
      digitalWrite(pinNr[i], 0);
      //usleep(uPause);
    }


    for(i=27; i>-1; i--){
      digitalWrite(pinNr[i], 1);
      usleep(uPause);
      digitalWrite(pinNr[i], 0);
      //usleep(uPause);
    }


    //Lightshow
    for(j=1; j<28; j++){
      uPause = 28000 - j * 1000;
      for(i=0; i<28+j; i++){
        for(k=0; k<j; k++){
          if(i-k > 0 && i-k < 28){
            digitalWrite(pinNr[i-k], 1);
          }
        }
        usleep(uPause);
        for(k=0; k<j; k++){
          if(i-k > 0 && i-k < 28){
            digitalWrite(pinNr[i-k], 0);
          }
        }
        //usleep(uPause);
      }


      for(i=27; i>-1-j; i--){
        for(k=0; k<j; k++){
          if(i+k < 28 && i+k > -1){
            digitalWrite(pinNr[i+k], 1);
          }
        }
        usleep(uPause);
        for(k=0; k<j; k++){
          if(i+k < 28 && i+k > -1){
            digitalWrite(pinNr[i+k], 0);
          }
        }
        //usleep(uPause);
      }
    }

  }


  return (0);


}




